<?php
/**
 * -----------------------------------------
 * @since 2.0 - 17.02.2025
 * @version 2.0
 * @link https://www.php-space.info/umfrage-script/
 * @author Nico Schubert / SchubertMedia
 * @abstract E-Mail: info@schubertmedia.de
 *
 * @copyright Es ist untersagt den Copyright Hinweis zu entfernen bei den Umfrage Script.
 * @abstract Eine Copyright freie Version können Sie für einmalig 20,- EUR inkl. 19% MwSt. erwerben,
 * hierzu möchten wir Sie bitten das Bestellformular https://www.php-space.info/script-erweiterung-bestellen/ zu nutzen.
 * -----------------------------------------
 */

/**
 * Berechnet den Prozentsatz auf Grundlage der übergebenen Stimmenanzahl und der Gesamtanzahl.
 *
 * Diese Funktion nimmt als Eingabe die Anzahl der Stimmen und die Gesamtanzahl (z. B. von möglichen Wählern)
 * und berechnet den Prozentsatz. Wenn die Gesamtanzahl oder die Anzahl der Stimmen 0 oder negativ ist,
 * gibt die Funktion 0 zurück, da dies eine ungültige Eingabe darstellt.
 *
 * **Ablauf der Funktion:**
 * 1. Überprüfung: Falls die Anzahl der Stimmen (`$votes`) oder die Gesamtanzahl (`$gesamt_anzahl`) kleiner
 *    oder gleich Null ist, wird 0.0 als Ergebnis zurückgegeben.
 * 2. Prozentsatzberechnung: Andernfalls wird der Prozentsatz berechnet:
 *    `(Anzahl der Stimmen / Gesamtanzahl) * 100`.
 * 3. Das Ergebnis wird als Fließkommazahl (float) zurückgegeben.
 *
 * Beispiel für den Aufruf:
 * <code>
 * echo prozent(25, 100); // Ausgabe: 25.0 (25%)
 * echo prozent(0, 50);   // Ausgabe: 0.0 (ungültige Eingabe)
 * echo prozent(-5, 60);  // Ausgabe: 0.0 (ungültige Eingabe)
 * echo prozent(30, 200); // Ausgabe: 15.0 (15%)
 * </code>
 *
 * @param int $votes        Anzahl der erhaltenen Stimmen (≥ 0).
 * @param int $gesamt_anzahl Gesamtanzahl (≥ 0).
 * @return float            Prozentsatz der Stimmen (z. B. 25.0 für 25%) oder 0, falls Eingaben ungültig sind.
 */
function prozent(int $votes, int $gesamt_anzahl): float
{
  // Wenn die Gesamtanzahl 0 ist oder keine Stimmen vorhanden sind, direkt 0 zurückgeben
  if ($votes <= 0 || $gesamt_anzahl <= 0) {
    return 0.0;
  }
  // Prozentsatz berechnen
  return ($votes / $gesamt_anzahl) * 100;
}

/**
 * Überprüft, ob eine IP-Adresse kürzlich eine bestimmte Aktion durchgeführt hat, und fügt die IP-Adresse bei Bedarf zur Datei hinzu.
 *
 * Diese Funktion wird verwendet, um doppelte oder zu häufige Aktionen zu verhindern, z. B. bei Umfragen oder ähnlichen Anwendungen.
 *
 * **Ablauf:**
 * 1. Die Funktion prüft, ob die festgelegte Datei `DATEI_IPS` existiert. Falls nicht, gibt sie `0` zurück.
 * 2. Falls die Konstante `IP_RELOAD` aktiviert ist, wird die Datei geöffnet.
 *    - Die IP-Adressen und ihre zugehörigen Zeitstempel werden gelesen und gefiltert.
 *    - Gültige Einträge werden basierend auf ihrer Relativität zu `ZEIT_RELOAD` beibehalten.
 *    - Stimmt die aktuelle IP-Adresse mit einem vorliegenden Eintrag überein, wird der Wert `$gefunden` auf `1` gesetzt.
 * 3. Die aktuelle IP-Adresse und der aktuelle Zeitstempel werden der Liste hinzugefügt.
 * 4. Der neue Inhalt wird in die Datei zurückgeschrieben und die Datei geschlossen.
 * 5. Die Funktion gibt zurück, ob die IP-Adresse bereits gefunden wurde (`1`) oder nicht (`0`).
 *
 * **Parameter:**
 * @param string $ip Die zu überprüfende IP-Adresse, z. B. `192.168.1.1`.
 *
 * **Rückgabewert:**
 * @return int Gibt `1` zurück, wenn die IP-Adresse bereits vorhanden ist, und `0`, wenn sie nicht gefunden wurde.
 *
 * **Konstanten:**
 * - `DATEI_IPS`: Der Dateipfad zur Datei, die alle IP-Adressen speichert.
 * - `IP_RELOAD`: Ein Schalter, der festlegt, ob ein erneuter Zugriff innerhalb einer bestimmten Zeit erlaubt ist.
 * - `ZEIT_RELOAD` (in Minuten): Die Zeitspanne, die vergehen muss, bevor eine IP-Adresse erneut gespeichert werden darf.
 *
 * **Anwendungsbeispiel:**
 * <code>
 * define('DATEI_IPS', 'ips.txt'); // Datei, die die IP-Adressen speichert
 * define('IP_RELOAD', true);      // Aktivierung der IP-Speicherung
 * define('ZEIT_RELOAD', 5);       // Wartezeit von 5 Minuten
 *
 * $ipAdresse = $_SERVER['REMOTE_ADDR'];
 *
 * if (check_ip($ipAdresse) === 0) {
 *     echo 'Die IP-Adresse wurde hinzugefügt.';
 * } else {
 *     echo 'Die IP-Adresse existiert bereits.';
 * }
 * </code>
 */
function check_ip(string $ip): int
{
  if (!file_exists(DATEI_IPS)) {
    return 0;
  }
  $gefunden = 0;
  if (IP_RELOAD) {
    $zeit_jetzt = time();
    $reload_dat = fopen(DATEI_IPS, "cb");
    if (!$reload_dat) {
      return 0; // Datei konnte nicht geöffnet werden
    }
    // Lies die Datei und filtere die gültigen IPs
    $ip_array = array_filter(array_map('trim', file(DATEI_IPS)), 'strlen');
    $neue_inhalte = [];
    foreach ($ip_array as $zeile) {
      [$ip_daten, $zeit_array] = explode("|", $zeile);
      $zeit_array = intval($zeit_array);
      // Nur gültige IPs beibehalten
      if ($zeit_jetzt < ($zeit_array + 60 * ZEIT_RELOAD)) {
        if ($ip_daten === $ip) {
          $gefunden = 1;
        }
        $neue_inhalte[] = $ip_daten . '|' . $zeit_array;
      }
    }
    // Füge die aktuelle IP hinzu
    $neue_inhalte[] = $ip . '|' . $zeit_jetzt;
    // Schreibe die aktualisierten Inhalte zurück in die Datei
    ftruncate($reload_dat, 0); // Datei leeren
    rewind($reload_dat); // Zurück zum Anfang der Datei
    fwrite($reload_dat, implode(PHP_EOL, $neue_inhalte) . PHP_EOL);
    fclose($reload_dat);
  }
  return $gefunden;
}